/*
 * Copyright 2006 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.google.gwt.user.client;

import com.google.gwt.core.client.JavaScriptObject;

/**
 * Provides access to browser cookies stored on the client.  Because of browser
 * restrictions, you will only be able to access cookies associated with the
 * current page's domain.
 */
public class Cookies {

  /**
   * Gets the cookie associated with the given key.
   * 
   * @param key the key of the cookie to be retrieved
   * @return the cookie's value.
   */
  public native String getCookie(String key) /*-{
    var cookies = this.@com.google.gwt.user.client.Cookies::loadCookies()();
    var value = cookies[key];
    if (value)
      return value;
    else
      return null;
  }-*/;

  native JavaScriptObject loadCookies() /*-{
    var cookies = {};

    var docCookie = $doc.cookie;
    if (docCookie && docCookie != '') {
      var crumbs = docCookie.split('; ');
      for (var i = 0; i < crumbs.length; ++i) {
        var parts = crumbs[i].split("=");
        if (parts.length == 2) {
          cookies[parts[0]] = unescape(parts[1]);
        }
      }
    }

    return cookies;
  }-*/;
}
